# OP 2019 - Problem 4: peak to peak

# Copywrite 2019 Robert Hasker

import math

def sqr(x):
    return x * x

def distance_to(start: (float, float, float), destination: (float, float, float)) -> float:
    (xs, ys, hs) = start
    (xd, yd, hd) = destination
    ground = math.sqrt(sqr((xd - xs) * 1000) + sqr((yd - ys) * 1000))
    gain = hd - hs
    total = math.sqrt(sqr(ground) + sqr(gain))
    return total / 1000

def pathlen(route):
    tot = 0.0
    for i in range(len(route) - 1):
        tot += distance_to(route[i], route[i + 1])
    if len(route) > 0:
        tot += distance_to(route[len(route) - 1], route[0])
    return tot

def greedy_path(start: (float, float, float), to_visit: set) -> float:
    location = start
    dist = 0.0
    while len(to_visit) > 0:
        best = min(to_visit, key= lambda x: distance_to(location, x))
        to_visit -= {best}
        if len(to_visit) > 1:
            next_best = min(to_visit, key= lambda x: distance_to(location, x))
            dist_to_best = distance_to(location, best)
            dist_to_alt  = distance_to(location, next_best)
            if abs(dist_to_best - dist_to_alt) < 0.001:
                print("Two close distances: " + str(dist_to_best) + ", " + str(dist_to_alt))
        dist += distance_to(location, best)
        location = best
    dist += distance_to(location, start)
    return dist

def dump(peaks: set):
    print("Peaks:")
    for p in peaks:
        (n, e, h) = p
        print("  (" + str(n) + "N, " + str(e) + "E, " + str(h) + " high)")

def main():
    filename = input("Enter peak location file: ")
    file = open(filename, 'r') 
    peaks = set()
    for line in file:
        (n_str, e_str, h_str) = line.split()
        peaks.add((float(n_str), float(e_str), float(h_str)))
    start = (0, 0, 0)
    path_len = greedy_path(start, peaks)
    print("Greedy route length: %.4f km" % path_len)

main()
